/**************************************************************/
/*   prepocess.c					      */
/*   Hasib Siddiqui                     		      */
/*   Jan 31, 2005	                      		      */
/**************************************************************/

#include "rs_denoise.h"

/* filter routines */

static float **build_gauss_filter(
  int 	filt_window,
  float filt_st_dev);

static float **build_gauss_filter ( 
  int filt_window,
  float filt_st_dev )
{

  int         i,j;
  float      **filter;
  double      two_sigma_squared, sum_of_squares,sum;

  /* Allocate filter and put pointer in middle of array */
  filter = (float **) 
    multialloc ( sizeof ( double ),2, 2*filt_window+1, 2*filt_window+1 );

  for ( i=0; i < 2*filt_window+1; i++ )
    filter[i] += filt_window;
  filter += filt_window;

  two_sigma_squared = 2.0*filt_st_dev*filt_st_dev;

  sum = 0;
  for (i=-filt_window;i<=filt_window;i++)
    for (j=-filt_window;j<=filt_window;j++) {
      sum_of_squares = ( double ) ( i*i + j*j );
      filter[i][j] = exp ( - sum_of_squares / two_sigma_squared );
      sum += filter[i][j];
    }

  /* Normalize filter */
  for (i=-filt_window;i<=filt_window;i++){
    for (j=-filt_window;j<=filt_window;j++){
      filter[i][j] /= sum;
    }
  }

  return(filter);
}

void apply_gauss_filter(
  struct pxm_img *in_img,
  struct pxm_img *out_img,
  float  std_dev,
  int 	 mask_size)

{

  int   	i, j, k, m, n, ht, wd, num_planes;
  float 	total, **filter;
  unsigned char **out, **in;

  ht = in_img->height;
  wd = in_img->width;

  /* determine if the input is type 'g' or 'p' */
  if(in_img->pxm_type == 'p'){
    num_planes = 3;
    get_pxm(out_img, ht, wd, 'p');
  }
  else{
    num_planes = 1;
    get_pxm(out_img, ht, wd, 'g'); 
  }

  /* Build the Gaussian Filter */
  filter  = build_gauss_filter (mask_size, std_dev);
	
  /* process the non-boundary pixels */
  for(k = 0; k < num_planes; k++){
    if(in_img->pxm_type == 'p'){
      in  = in_img->color[k];
      out = out_img->color[k];
    }
    else{
      in  = in_img->mono;
      out = out_img->mono;
    }

    for(i = mask_size; i < ht - mask_size; i++)
      for(j = mask_size; j < wd - mask_size; j++) {
	total  = 0;
	for(m = -mask_size; m <= mask_size; m++)
	  for(n = -mask_size; n <= mask_size; n++)
            total += filter[m][n] * (in[i + m][j + n]);
	out[i][j] = total;
      }

#if 0
    for(i = mask_size; i < ht - mask_size; i++)
      for(j = mask_size; j < wd - mask_size; j++) {
        col_sum = 0.0;
        for(m = -mask_size; m <= mask_size; m++){
	  row_sum = 0.0;
          for(n = -mask_size; n <= mask_size; n++)
            row_sum += filter1[n+1] * (float)(in[i + m][j + n]);
        col_sum += row_sum * filter1[m+1];
      }
    out[i][j] = (unsigned char)col_sum; 
    }
#endif

    /* take care of the boundary pixels */
    for(i = 0; i < mask_size; i++)
      for(j = mask_size; j < wd - mask_size; j++){
	/* top */
	out[i][j] = out[mask_size][j];
	/* bottom */
	out[i + ht - mask_size][j] =
	  out[i + ht - mask_size - 1][j];
      }
    
    for(i = 0; i < ht; i++)
      for(j = 0; j < mask_size; j++){
	/* left */
	out[i][j] = out[i][mask_size];
	/* right */
	out[i][j + wd - mask_size] =
	  out[i][j + wd - mask_size - 1];
      }
  }
}

